/*
 * Recon allows monitoring and recording of data stream trough serial port.
 * Copyright (C) 2008-2009  Linas Žvirblis <0x0007@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package lt.ku.ik.recon;

import java.awt.Image;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.UIManager;
import lt.ku.ik.recon.data.global.Constants;
import lt.ku.ik.recon.logic.service.ConstantsService;
import lt.ku.ik.recon.logic.service.LocaleService;
import lt.ku.ik.recon.logic.service.SerialPortService;
import lt.ku.ik.recon.logic.service.SettingsService;
import lt.ku.ik.recon.presentation.gui.widgets.MainWindowFrame;

/**
 *
 * @author linas
 */
public class Application {

    void run() {

        SettingsService settingsService = new SettingsService();
        LocaleService localeService = new LocaleService();
        SerialPortService serialPortService = new SerialPortService(settingsService);
        ConstantsService constantsService = new ConstantsService();

        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception exception) {
            exception.printStackTrace();
        }

        MainWindowFrame mainWindowFrame = new MainWindowFrame();
        ArrayList<Image> imageIconList = new ArrayList<Image>();

        ImageIcon imageIconSmall = new ImageIcon(getClass().getResource("/lt/ku/ik/recon/presentation/gui/images/buttons/graph-small.png"));
        ImageIcon imageIconBig = new ImageIcon(getClass().getResource("/lt/ku/ik/recon/presentation/gui/images/buttons/graph.png"));

        Image imageSmall = imageIconSmall.getImage();
        Image imageBig = imageIconBig.getImage();

        imageIconList.add(imageSmall);
        imageIconList.add(imageBig);

        mainWindowFrame.setLocaleService(localeService);
        mainWindowFrame.setSettingsService(settingsService);
        mainWindowFrame.setSerialPortService(serialPortService);

        mainWindowFrame.setTitle(constantsService.getConstant("APP_NAME"));
        mainWindowFrame.setIconImages(imageIconList);
        mainWindowFrame.setupPanels();
        mainWindowFrame.setupListeners();
        mainWindowFrame.pack();
        mainWindowFrame.setVisible(true);
    }
}
