/*
 * Recon allows monitoring and recording of data stream trough serial port.
 * Copyright (C) 2008-2009  Linas Žvirblis <0x0007@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package lt.ku.ik.recon.data.comm;

import lt.ku.ik.recon.logic.model.DataException;
import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.NoSuchPortException;
import gnu.io.SerialPort;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashSet;

public class SerialPortDao {

    private SerialPort serialPort;

    public SerialPort getSerialPort() {
        return serialPort;
    }

    public HashSet<CommPortIdentifier> getAvailablePorts() {

        HashSet<CommPortIdentifier> commPortHashSet = new HashSet<CommPortIdentifier>();
        Enumeration portEnumeration = CommPortIdentifier.getPortIdentifiers();

        while (portEnumeration.hasMoreElements()) {

            CommPortIdentifier commPortIdentifier = (CommPortIdentifier) portEnumeration.nextElement();

            if ((commPortIdentifier.getPortType() == CommPortIdentifier.PORT_SERIAL) &&
                    (!commPortIdentifier.isCurrentlyOwned())) {

                commPortHashSet.add(commPortIdentifier);
            }
        }

        return commPortHashSet;
    }

    public void connect(
            int baudRate, int dataBits, int stopBits, int parity)
            throws DataException {

        CommPortIdentifier commPortIdentifier = getAvailablePorts().iterator().next();
        connect(commPortIdentifier, baudRate, dataBits, stopBits, parity);
    }

    public void connect(
            String portName,
            int baudRate, int dataBits, int stopBits, int parity)
            throws DataException {

        CommPortIdentifier commPortIdentifier = null;

        try {
            commPortIdentifier = CommPortIdentifier.getPortIdentifier(portName);
        } catch (NoSuchPortException exception) {
            throw new DataException(exception);
        }

        connect(commPortIdentifier, baudRate, dataBits, stopBits, parity);
    }

    public void connect(
            CommPortIdentifier commPortIdentifier,
            int baudRate, int dataBits, int stopBits, int parity)
            throws DataException {

        try {
            CommPort commPort = commPortIdentifier.open(this.getClass().getName(), 2000);
            serialPort = (SerialPort) commPort;
            serialPort.setSerialPortParams(baudRate, dataBits, stopBits, parity);
        } catch (Exception exception) {
            throw new DataException(exception);
        }
    }

    public void disconnect() {
        serialPort.close();
    }

    public void sendCommand(int command) throws DataException {
        try {
            serialPort.getOutputStream().write(command);
        } catch (IOException exception) {
            throw new DataException(exception);
        }
    }

//    private void setupListeners() throws DataException {
//
//        SerialPortEventListener serialEventListener = new SerialPortEventListener() {
//
//            @Override
//            public synchronized void serialEvent(SerialPortEvent serialPortEvent) {
//                // readSerialPort(serialPortEvent);
//            }
//        };
//
//        try {
//            serialPort.addEventListener(serialEventListener);
//            serialPort.notifyOnDataAvailable(true);
//        } catch (TooManyListenersException exception) {
//            throw new DataException(exception);
//        }
//    }
}
