/*
 * Recon allows monitoring and recording of data stream trough serial port.
 * Copyright (C) 2008-2009  Linas Žvirblis <0x0007@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package lt.ku.ik.recon.logic.service;

import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import java.util.HashSet;
import lt.ku.ik.recon.data.comm.SerialPortDao;
import lt.ku.ik.recon.logic.model.DataException;
import lt.ku.ik.recon.logic.model.ServiceException;
import lt.ku.ik.recon.logic.model.Settings;

/**
 *
 * @author linas
 */
public class SerialPortService {

    private SettingsService settingsService;
    private SerialPortDao serialPortDao;

    public SerialPortService(SettingsService settingsService) {
        this.settingsService = settingsService;

        if (serialPortDao == null) {
            serialPortDao = new SerialPortDao();
        }
    }

    public void setSettingsService(SettingsService settingsService) {
        this.settingsService = settingsService;
    }

    public SerialPort getSerialPort() {
        return serialPortDao.getSerialPort();
    }

    public HashSet<CommPortIdentifier> getAvailablePorts() {
        return serialPortDao.getAvailablePorts();
    }

    public void startTransmission() throws ServiceException {
        try {
            serialPortDao.sendCommand(settingsService.getSettings().getStartCommand());
        } catch (DataException exception) {
            throw new ServiceException(exception);
        }
    }

    public void stopTransmission() throws ServiceException {
        try {
            serialPortDao.sendCommand(settingsService.getSettings().getStopCommand());
        } catch (DataException exception) {
            throw new ServiceException(exception);
        }
    }

    public void connect() throws ServiceException {

        Settings settings = settingsService.getSettings();

        if (settingsService.getSettings().getDevice() != null) {
            try {
                serialPortDao.connect(
                        settings.getDevice(),
                        settings.getBaudRate().rate(),
                        settings.getDataBits().bits(),
                        settings.getStopBits().bits(),
                        settings.getParity().parity());
            } catch (DataException exception) {
                throw new ServiceException(exception);
            }
        } else {
            try {
                serialPortDao.connect(
                        settings.getBaudRate().rate(),
                        settings.getDataBits().bits(),
                        settings.getStopBits().bits(),
                        settings.getParity().parity());
            } catch (DataException exception) {
                throw new ServiceException(exception);
            }
        }
    }

    public void disconnect() {
        serialPortDao.disconnect();
    }
}
