/*
 * Recon allows monitoring and recording of data stream trough serial port.
 * Copyright (C) 2008-2009  Linas Žvirblis <0x0007@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package lt.ku.ik.recon.presentation.gui.widgets;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.SystemColor;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import lt.ku.ik.recon.logic.service.SettingsService;

/**
 *
 * @author linas
 */
public class DrawingPanel extends JPanel {

    private SettingsService settingsService;
    private int[] rawBuffer;
    private boolean graphDrawn = true;

    public DrawingPanel() {
        Border border = new EtchedBorder(EtchedBorder.LOWERED);
        setBorder(border);
        setDoubleBuffered(true);
    }

    public void setSettingsService(SettingsService settingsService) {
        this.settingsService = settingsService;
    }

    public void setBuffer(int[] buffer) {
        this.rawBuffer = buffer;
    }

    public void setGraphDrawn(boolean graphDrawn) {
        this.graphDrawn = graphDrawn;
    }

    public boolean isGrapDrawn() {
        return graphDrawn;
    }

    @Override
    public void paintComponent(Graphics graphics) {

        super.paintComponent(graphics);
        drawBackground(graphics);
        drawGrid(graphics);

        if (graphDrawn && (rawBuffer != null)) {
            drawGraph(graphics);
        }
    }

    private int normalizeHeight(int height) {

        Dimension dimension = getSize();
        Double accuracy = Double.valueOf(settingsService.getMaxValue());
        Double aspect = dimension.height / accuracy;

        int normalizedHeight = (int) (dimension.height - (height * aspect));
        return normalizedHeight;
    }

    private int normalizeWidth(int width) {

        Dimension dimension = getSize();
        Double resolution = Double.valueOf(settingsService.getSettings().getResolutionOfGraph());
        Double aspect = dimension.width / resolution;

        int normalizedWidth = (int) (width * aspect);

        return normalizedWidth;
    }

    private void drawBackground(Graphics graphics) {
        graphics.setColor(SystemColor.text);
        graphics.fillRect(0, 0, getSize().width, getSize().height);
    }

    private void drawGrid(Graphics graphics) {

        int accuracy = settingsService.getMaxValue();
        int resolution = settingsService.getSettings().getResolutionOfGraph();

        int verticalStep = resolution / 200;
        int horizontalStep = accuracy / 4;

        graphics.setColor(SystemColor.control);

        for (int i = verticalStep; i < resolution; i = i + verticalStep) {
            graphics.drawLine(
                    normalizeWidth(i),
                    normalizeHeight(0),
                    normalizeWidth(i),
                    normalizeHeight(accuracy));
        }

        for (int i = horizontalStep; i < accuracy; i = i + horizontalStep) {
            graphics.drawLine(
                    normalizeWidth(0),
                    normalizeHeight(i),
                    normalizeWidth(resolution),
                    normalizeHeight(i));
        }
    }

    private int[] getSequenceNumbers() {

        int resolution = settingsService.getSettings().getResolutionOfGraph();
        int[] sequence = new int[resolution];

        for (int i = 0; i < sequence.length; i++) {
            sequence[i] = normalizeWidth(i);
        }

        return sequence;
    }

    private void drawGraph(Graphics graphics) {

        int[] sequence = getSequenceNumbers();
        int[] normalizedBuffer = new int[rawBuffer.length];

        for (int i = 0; i < rawBuffer.length; i++) {
            normalizedBuffer[i] = normalizeHeight(rawBuffer[i]);
        }

        graphics.setColor(SystemColor.red);
        graphics.drawPolyline(sequence, normalizedBuffer, rawBuffer.length);
    }
}
