/*
 * Recon allows monitoring and recording of data stream trough serial port.
 * Copyright (C) 2008-2009  Linas Žvirblis <0x0007@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package lt.ku.ik.recon.presentation.gui.widgets;

import gnu.io.CommPortIdentifier;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Iterator;
import lt.ku.ik.recon.logic.model.BaudRate;
import lt.ku.ik.recon.logic.model.CommandFormat;
import lt.ku.ik.recon.logic.model.DataBits;
import lt.ku.ik.recon.logic.model.Parity;
import lt.ku.ik.recon.logic.model.Settings;
import lt.ku.ik.recon.logic.model.StopBits;
import lt.ku.ik.recon.logic.service.SerialPortService;
import lt.ku.ik.recon.logic.service.SettingsService;

/**
 *
 * @author linas
 */
public class SettingsWindowFrame extends SettingsWindowSketch {

    private SettingsService settingsService;
    private SerialPortService serialPortService;

    public void setSettingsService(SettingsService settingsService) {
        this.settingsService = settingsService;
    }

    public void setSerialPortService(SerialPortService serialPortService) {
        this.serialPortService = serialPortService;
    }

    public void setupListeners() {

         ActionListener closeActionListener = new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                dispose();
            }
        };

        jButtonClose.addActionListener(closeActionListener);

         ActionListener applyActionListener = new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                apply();
            }
        };

        jButtonApply.addActionListener(applyActionListener);
    }

    public void setValues() {

        Settings settings = settingsService.getSettings();
        HashSet<CommPortIdentifier> devices = serialPortService.getAvailablePorts();
        Iterator deviceIterator = devices.iterator();

        while (deviceIterator.hasNext()) {
            CommPortIdentifier commPortIdentifier = (CommPortIdentifier) deviceIterator.next();
            String port = commPortIdentifier.getName();
            jComboBoxDevice.addItem(port);
        }

        for (BaudRate baudRate : BaudRate.values()) {
            jComboBoxBaudRate.addItem(baudRate);
        }

        for (DataBits dataBits : DataBits.values()) {
            jComboBoxDataBits.addItem(dataBits);
        }

        for (StopBits stopBits : StopBits.values()) {
            jComboBoxStopBits.addItem(stopBits);
        }

        for (Parity parity : Parity.values()) {
            jComboBoxParity.addItem(parity);
        }

        for (CommandFormat commandFormat : CommandFormat.values()) {
            jComboBoxCommandFormat.addItem(commandFormat);
        }

        jComboBoxDevice.setSelectedItem(settings.getDevice());
        jComboBoxBaudRate.setSelectedItem(settings.getBaudRate());
        jComboBoxDataBits.setSelectedItem(settings.getDataBits());
        jComboBoxStopBits.setSelectedItem(settings.getStopBits());
        jComboBoxParity.setSelectedItem(settings.getParity());

        jSpinnerPrecision.setValue(settings.getPrecision());
        jSpinnerChannels.setValue(settings.getChannels());

        jTextFieldStartCommand.setText(String.valueOf(settings.getStartCommand()));
        jTextFieldStopCommand.setText(String.valueOf(settings.getStopCommand()));

        jComboBoxCommandFormat.setSelectedItem(settings.getCommandFormat());
        jComboBoxCommandFormat.setEnabled(false);
    }

    public void apply() {
        Settings settings = settingsService.getSettings();

        settings.setDevice(String.valueOf(jComboBoxDevice.getSelectedItem()));
        settings.setBaudRate((BaudRate) jComboBoxBaudRate.getSelectedItem());
        settings.setDataBits((DataBits) jComboBoxDataBits.getSelectedItem());
        settings.setStopBits((StopBits) jComboBoxStopBits.getSelectedItem());
        settings.setParity((Parity) jComboBoxParity.getSelectedItem());

        settings.setPrecision((Integer) jSpinnerPrecision.getValue());
        settings.setChannels((Integer) jSpinnerChannels.getValue());

        settings.setStartCommand(Integer.valueOf(jTextFieldStartCommand.getText()));
        settings.setStopCommand(Integer.valueOf(jTextFieldStopCommand.getText()));

        settings.setCommandFormat((CommandFormat) jComboBoxCommandFormat.getSelectedItem());
    }
}
